"""
Create an interactive python shell with the given files opened for
reading.

::

   usage: read.py [-h] ...

   Interactive getar-format archive shell

   positional arguments:
     inputs      Input files to open

   optional arguments:
     -h, --help  show this help message and exit
"""

import argparse
import code
import os

import gtar

parser = argparse.ArgumentParser(
    description='Interactive getar-format archive shell')
parser.add_argument('inputs', nargs=argparse.REMAINDER,
                    help='Input files to open')

messageTemplate = """
This is a python interactive shell.

{}
"""

definitions = {
    'archives': 'a list of all input files, opened for reading'
    }

def main(inputs):
    archives = [gtar.GTAR(path, 'r') for path in inputs]

    if len(inputs) == 1:
        traj = archives[0]
        definitions['traj'] = 'the opened input file'
        records = {rec.getName(): rec for rec in traj.getRecordTypes()}
        definitions['records'] = 'a dictionary of available records, indexed by property name'
        recordFrames = {name: traj.queryFrames(records[name]) for name in records}
        definitions['recordFrames'] = 'a dictionary of available frames for each property by name'

    message = messageTemplate.format(
        '\n'.join('- {} is {}.'.format(k, definitions[k]) for k in sorted(definitions)))

    code.interact(banner=message, local=dict(globals(), **locals()))

if __name__ == '__main__': main(**vars(parser.parse_args()))



import random
import sys
import gtar

def randomPath(term=.25):
    """Generates a random (but valid) path name for a property given a
    chance to terminate after any segment"""
    validNameChars = ([chr(c) for c in range(ord('a'), ord('z')+1)] +
                      [chr(c) for c in range(ord('A'), ord('A')+1)] +
                      [chr(c) for c in range(ord('0'), ord('9')+1)] + ['.', '/'])
    meaningfulBits = ['frames', 'vars', '.f32', '.f64', '.u8', '.u64', '.i32',
                      '.i64', '.ind', '.uni']
    randomBits = [''.join(random.sample(validNameChars, random.randint(1, 10))) for _ in range(20)]
    allBits = meaningfulBits + randomBits

    result = ''

    while random.random() > term:
        seg = ''.join(random.sample(allBits, random.randint(1, 5)))
        result += seg + '/'

    # get rid of trailing slash
    result = result[:-1]

    return (result if result else randomPath(term=term))

def test_pathIdentity(n=100):
    """Creating a Record object from a reasonable path and asking for
    its path should always be identity"""
    success = True
    for _ in range(n):
        path = randomPath()
        rec = gtar.Record(path)
        reconstructedPath = rec.getPath()

        relPath = path.lstrip('/')

        if relPath != reconstructedPath:
            print('Error reconstructing path from Record!'
                  ' "{}" != "{}" [{}, {}, {}]'.format(
                      relPath, reconstructedPath, rec.getGroup(), rec.getName(), rec.getIndex()))
            success = False

    assert success
    return success

def test_pathKnown():
    success = True

    (ind, uni) = (gtar.Resolution.Individual, gtar.Resolution.Uniform)
    txt = gtar.Resolution.Text
    (f32, u8, u32) = (gtar.Format.Float32, gtar.Format.UInt8, gtar.Format.UInt32)

    recs = {
        ('', 'position', '1000', f32, ind): gtar.Record('frames/1000/position.f32.ind'),
        ('rigid', 'body', '', u32, ind): gtar.Record('rigid/body.u32.ind'),
        ('rigid', 'com', 'ab cd', f32, ind): gtar.Record('rigid/frames/ab cd/com.f32.ind'),
        ('', 'N', '', u32, uni): gtar.Record('N.u32.uni'),
        ('', 'test.txt', '', u8, txt): gtar.Record('test.txt'),
        ('hoomd', 'types.json', '1024', u8, txt): gtar.Record('hoomd/frames/1024/types.json')
        }

    for key in recs:
        (group, name, index, fmt, res) = key
        rec = recs[key]

        match = True
        match = match and rec.getGroup() == group
        match = match and rec.getName() == name
        match = match and rec.getIndex() == index
        match = match and rec.getFormat() == fmt
        match = match and rec.getResolution() == res

        if not match:
            print('Error reconstructing known path {} (reconstructed path '
                  '"{}")!'.format(key, rec.getPath()))

        success = success and match

    assert success
    return success

def main():
    success = True

    for test in [test_pathIdentity, test_pathKnown]:
        try:
            success = test() and success
        except AssertionError:
            success = False

    sys.exit(not success)

if __name__ == '__main__': main()



"""
Take records from multiple getar-formatted files and place them
into an output file. In case of name conflicts, records from the last
input file take precedence.

::

   usage: cat.py [-h] [-o OUTPUT] ...

   Command-line archive concatenation

   positional arguments:
     inputs                Input files to read

   optional arguments:
     -h, --help            show this help message and exit
     -o OUTPUT, --output OUTPUT
                           File to write to
"""

import argparse
from collections import defaultdict
import os

import gtar

parser = argparse.ArgumentParser(
    description='Command-line archive concatenation')
parser.add_argument('inputs', nargs=argparse.REMAINDER,
                    help='Input files to read')
parser.add_argument('-o', '--output', required=True,
                    help='File to write to')

def main(inputs, output):
    """Take all records from a set of getar-formatted files and output them to another.

    :param inputs: Input filenames to concatenate
    :param output: Output filename (can be the same as input)

    """
    nameHalves = os.path.splitext(output)
    tempName = output

    while os.path.exists(tempName):
        nameHalves = (nameHalves[0] + '_', nameHalves[1])
        tempName = nameHalves[0] + nameHalves[1]

    recordFiles = {}
    for input in inputs:
        with gtar.GTAR(input, 'r') as inpFile:
            recs = {rec: inpFile.queryFrames(rec) for rec in inpFile.getRecordTypes()}

            for rec in recs:
                frames = recs[rec]
                for frame in frames:
                    rec.setIndex(frame)
                    recordFiles[rec.getPath()] = input

    sourceRecords = defaultdict(list)
    for path in recordFiles:
        sourceRecords[recordFiles[path]].append(path)

    try:
        with gtar.GTAR(tempName, 'w') as out:
            for source in sourceRecords:
                allPaths = sorted(sourceRecords[source])
                with gtar.GTAR(source, 'r') as inp:
                    for path in allPaths:
                        out.writePath(path, inp.readPath(path))

        if not os.path.samefile(tempName, output):
            os.rename(tempName, output)
    finally:
        if os.path.exists(tempName) and not os.path.samefile(tempName, output):
            os.remove(tempName)

if __name__ == '__main__': main(**vars(parser.parse_args()))
